-----------------------------------------------------------------------------------------------
-- Client Lua Script for CostumeManager
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "Apollo"
require "ChatSystemLib"
require "CostumesLib"
require "GameLib"
 
-----------------------------------------------------------------------------------------------
-- CostumeManager Module Definition
-----------------------------------------------------------------------------------------------
local CostumeManager = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local ptrCostumeIndex = 1
local strIntervalTimerMinimum = "0.6"
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function CostumeManager:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

	-- Status
	self.bRunning = false

	-- Mode
	self.bSequence = false;
	self.bRandom = false;
	
	-- AutoStart
	self.bAutoStart = false;
		
	-- Timer 
	self.strTimerInterval = 1.5;
	
	-- Costume Settings
	self.strCostumeSequence = "12345654321"
	self.strRandomOptions = "1234"

    return o
end

function CostumeManager:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- CostumeManager OnLoad
-----------------------------------------------------------------------------------------------
function CostumeManager:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("CostumeManager.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- CostumeManager OnDocLoaded
-----------------------------------------------------------------------------------------------
function CostumeManager:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "CostumeManagerWindow", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- Timer Registration
		Apollo.RegisterTimerHandler("CostumeManagerTimer", "OnCostumeManagerTimer", self)

		-- Listeners
		Apollo.RegisterEventHandler("GenericEvent_OpenCostumes", "OnGenericEvent_OpenCostumes", self)
		Apollo.RegisterEventHandler("GenericEvent_LoadItemPreview", "OnGenericEvent_LoadItemPreview", self)
		
		-- Slash Commands
		Apollo.RegisterSlashCommand("cm", "OnCostumeManagerOn", self)

	end
end

-----------------------------------------------------------------------------------------------
-- CostumeManager Functions
-----------------------------------------------------------------------------------------------
function CostumeManager:OnCostumeManagerOn()
	self.wndMain:Invoke()
	if self.strCostumeSequence ~= nil then
		self.wndMain:FindChild("CostumeSequenceInput"):SetText(self.strCostumeSequence)
	end
	
	if self.strRandomOptions ~= nil then
		self.wndMain:FindChild("RandomOptionsInput"):SetText(self.strRandomOptions)
	end

	if self.strTimerInterval ~= nil then
		self.wndMain:FindChild("TimerIntervalInput"):SetText(tostring(self.strTimerInterval ))
	end
	
	if self.bRandom ~= nil then
		self.wndMain:FindChild("RandomCheck"):SetCheck(self.bRandom)
	end
	
	if self.bSequence ~= nil then
		self.wndMain:FindChild("SequenceCheck"):SetCheck(self.bSequence)
	end
	
	if self.bAutoStart ~= nil then
		self.wndMain:FindChild("AutoStartCheck"):SetCheck(self.bAutoStart)
	end
end

function CostumeManager:OnCostumeManagerTimer()
	if self.bRunning then
		if self.bSequence then
			if ptrCostumeIndex > string.len(self.strCostumeSequence) then
				ptrCostumeIndex = 1
			end
			CostumesLib.SetCostumeIndex(tonumber(string.sub(self.strCostumeSequence, ptrCostumeIndex, ptrCostumeIndex)))
			ptrCostumeIndex = ptrCostumeIndex + 1
			
			--Check if the next costume index is a timer definition
			if(string.sub(self.strCostumeSequence, ptrCostumeIndex, ptrCostumeIndex) == "[") then
				-- If a delay is provided, set that up here.
				local dTimerDelay = tonumber(string.sub(self.strCostumeSequence, ptrCostumeIndex + 1, string.find(self.strCostumeSequence, "]", ptrCostumeIndex) - 1))
				if dTimerDelay < tonumber(strIntervalTimerMinimum) then
					dTimerDelay = tonumber(strIntervalTimerMinimum)
				end
				Apollo.StopTimer("CostumeManagerTimer")
				Apollo.CreateTimer("CostumeManagerTimer", dTimerDelay, true)
				ptrCostumeIndex = string.find(self.strCostumeSequence, "]", ptrCostumeIndex) + 1
			else
				Apollo.StopTimer("CostumeManagerTimer")
				Apollo.CreateTimer("CostumeManagerTimer", self.strTimerInterval, true)
			end
		elseif self.bRandom and string.len(self.strRandomOptions) > 0 then
			randomCostumeIndex = math.random(1, string.len(self.strRandomOptions))
			CostumesLib.SetCostumeIndex(tonumber(string.sub(self.strRandomOptions, randomCostumeIndex , randomCostumeIndex)))
		end
	end
end

function CostumeManager:SendChatMessage(strMessage)
	ChatSystemLib.PostOnChannel(2,"[CostumeManagerNotify] " .. strMessage)
end

function CostumeManager:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end
	tSave = {
		strTimerInterval = self.strTimerInterval ,
		strCostumeSequence = self.strCostumeSequence,
		strRandomOptions = self.strRandomOptions,
		bAutoStart = self.bAutoStart,
		bRandom = self.bRandom,
		bSequence = self.bSequence
	}
	return tSave
end

function CostumeManager:OnRestore(eType, tSave)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end

	if tSave ~= nil then
		if tonumber(tSave.strTimerInterval ) < tonumber(strIntervalTimerMinimum) then
			self.strTimerInterval = strIntervalTimerMinimum
		else 
			self.strTimerInterval = tSave.strTimerInterval 
		end
		self.strCostumeSequence =  tSave.strCostumeSequence 
		self.strRandomOptions = tSave.strRandomOptions 
		self.bAutoStart = tSave.bAutoStart
		self.bRandom = tSave.bRandom
		self.bSequence = tSave.bSequence
		
		if self.bAutoStart then
			self:RunWithCurrentConfig()
		end	
	end
end

-----------------------------------------------------------------------------------------------
-- CostumeManagerForm Functions
-----------------------------------------------------------------------------------------------
function CostumeManager:OnOK()
	self.bAutoStart = self.wndMain:FindChild("AutoStartCheck"):IsChecked()
	self.bRandom = self.wndMain:FindChild("RandomCheck"):IsChecked()
	self.bSequence = self.wndMain:FindChild("SequenceCheck"):IsChecked()
	self.strTimerInterval = self.wndMain:FindChild("TimerIntervalInput"):GetText()
	self.strCostumeSequence = self.wndMain:FindChild("CostumeSequenceInput"):GetText()
	self.strRandomOptions = self.wndMain:FindChild("RandomOptionsInput"):GetText()

	if tonumber(self.strTimerInterval) < tonumber(strIntervalTimerMinimum) then
		self.strTimerInterval = strIntervalTimerMinimum 
	end
	
	-- Settings
	self:SendChatMessage("Settings Updated!")
	
	-- Run
	self:RunWithCurrentConfig()

	self.wndMain:Close()
end

function CostumeManager:RunWithCurrentConfig()
	if self.bSequence or self.bRandom then
		self.bRunning = true
		Apollo.StopTimer("CostumeManagerTimer")
		Apollo.CreateTimer("CostumeManagerTimer", self.strTimerInterval, true)
	else 
		self.bRunning = false
		Apollo.StopTimer("CostumeManagerTimer")
	end
end

function CostumeManager:OnCancel()
	self.wndMain:Close()
end

---------------------------------------------------------------------------------------------------
-- CostumeManagerWindow Functions
---------------------------------------------------------------------------------------------------
function CostumeManager:OnOptionSelected( wndHandler, wndControl, eMouseButton )
	if self.wndMain:FindChild("SequenceCheck") == wndControl then
		self.wndMain:FindChild("RandomCheck"):SetCheck(false)
	elseif self.wndMain:FindChild("RandomCheck") == wndControl then
		self.wndMain:FindChild("SequenceCheck"):SetCheck(false)
	end
end

function CostumeManager:OnCostumeBtn( wndHandler, wndControl, eMouseButton )
	Event_FireGenericEvent("GenericEvent_OpenCostumes")
	-- Stop the addon while costumes are being modified.
	self:pause()
end

function CostumeManager:OnGenericEvent_OpenCostumes()
	self:pause()
end

function CostumeManager:OnGenericEvent_LoadItemPreview() 
	self:pause()
end

function CostumeManager:pause() 
	self.bRunning = false
	Apollo.StopTimer("CostumeManagerTimer")
end
-----------------------------------------------------------------------------------------------
-- CostumeManager Instance
-----------------------------------------------------------------------------------------------
local CostumeManagerInst = CostumeManager:new()
CostumeManagerInst:Init()
